#include <cmath>
#include "rectangle.h"

int CRectangle::count = 0;

// Конструктор за замовчуванням
CRectangle::CRectangle() : length(0), width(0) {
    count++; // збільшуємо лічильник при створенні нового об'єкта
}

// Конструктор для початкової ініціалізації з однаковими довжинами сторін
CRectangle::CRectangle(double side) : length(side), width(side) {
    count++;
}

// Конструктор для початкової ініціалізації з різними довжинами сторін
CRectangle::CRectangle(double l, double w) : length(l), width(w) {
    count++;
}

// деструктор
CRectangle::~CRectangle() {
    count--; // зменшуємо лічильник при знищенні об'єкта
}

// Функція для знаходження площі прямокутника
double CRectangle::calculateArea() const {
    return length * width;
}

// Перевантажена функція для знаходження площі прямокутника
double CRectangle::calculateArea(double l, double w)const {
    return l * w;
}

// Функція для знаходження периметру прямокутника
double CRectangle::calculatePerimeter() const {
    return 2 * (length + width);
}

// Перевантажена функція для знаходження периметру прямокутника
double CRectangle::calculatePerimeter(double l, double w) {
    return 2 * (l + w);
}

// Функція для знаходження співвідношення сторін
double CRectangle::aspectRatio() const {
    return length / width;
}

//Перевантажена функція для знаходження співвідношення сторін
double CRectangle::aspectRatio(double l, double w){
    return l/w;
}

// Функція для обертання прямокутника (зміна сторін)
void CRectangle::rotate() {
    double temp = length;
    length = width;
    width = temp;
}

// Перевантаження оператора додавання для збільшення прямокутника на константу
void CRectangle::operator+(double increment) {
    length += increment;
    width += increment;
}

// Перевантаження оператора множення для збільшення прямокутника у певну кількість разів
void CRectangle::operator*(double multiplier) {
    length *= multiplier;
    width *= multiplier;
}

// Функція для приведення прямокутника до дійсного типу (повертає його площу)
CRectangle::operator double() const {
    return calculateArea();
}

// Функція для обертання прямокутника (зміна сторін)
void CRectangle::operator!() {
    rotate();
}

// Функція для отримання довжини першої сторони
double CRectangle::getLength() const {
    return length;
}

// Функція для отримання довжини другої сторони
double CRectangle::getWidth() const {
    return width;
}

// Функція для встановлення нових значень сторін
void CRectangle::setSides(double l, double w) {
    length = l;
    width = w;
}

// Статична функція для отримання кількості створених об'єктів
int CRectangle::getCount() {
    return count;
}

// Перевантаження операторів як дружні функцій класу

// Введення прямокутника з потоку вводу
std::istream& operator>>(std::istream& in, CRectangle& rectangle) {
    double l, w;
    in >> l >> w;
    rectangle.setSides(l, w);
    return in;
}

// Виведення прямокутника в потік виводу
std::ostream& operator<<(std::ostream& out, const CRectangle& rectangle) {
    out << "Length: " << rectangle.getLength() << ", Width: " << rectangle.getWidth();
    return out;
}

// Перевірка рівності двох прямокутників
bool operator==(const CRectangle& rect1, const CRectangle& rect2) {
    return (rect1.getLength() == rect2.getLength() && rect1.getWidth() == rect2.getWidth());
}

// Перевірка нерівності двох прямокутників
bool operator!=(const CRectangle& rect1, const CRectangle& rect2) {
    return !(rect1 == rect2);
}
